# -*- coding: utf-8 -*-
import json

from django.core import mail
from django.core.checks import Info
from django.test import TestCase, override_settings

from wagtail.contrib.forms.models import FormSubmission
from wagtail.contrib.forms.tests.utils import (
    make_form_page, make_form_page_with_custom_submission, make_form_page_with_redirect,
    make_types_test_form_page)
from wagtail.models import Page
from wagtail.test.testapp.models import (
    CustomFormPageSubmission, ExtendedFormField, FormField, FormFieldWithCustomSubmission,
    FormPageWithCustomFormBuilder, JadeFormPage)
from wagtail.test.utils import WagtailTestUtils


class TestFormSubmission(TestCase):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page()

    def test_get_form(self):
        response = self.client.get('/contact-us/')

        # Check response
        self.assertContains(response, """<label for="id_your_email">Your email</label>""")
        self.assertTemplateUsed(response, 'tests/form_page.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_landing.html')

        # HTML in help text should be escaped
        self.assertContains(response, "&lt;em&gt;please&lt;/em&gt; be polite")

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

    @override_settings(WAGTAILFORMS_HELP_TEXT_ALLOW_HTML=True)
    def test_get_form_without_help_text_escaping(self):
        response = self.client.get('/contact-us/')
        # HTML in help text should not be escaped
        self.assertContains(response, "<em>please</em> be polite")

    def test_label_escaping(self):
        FormField.objects.filter(label="Your message").update(label="Your <em>wonderful</em> message")
        response = self.client.get('/contact-us/')
        self.assertContains(response, """<label for="id_your_message">Your &lt;em&gt;wonderful&lt;/em&gt; message</label>""")

    def test_post_invalid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob',
            'your_message': 'hello world',
            'your_choices': ''
        })

        # Check response
        self.assertContains(response, "Enter a valid email address.")
        self.assertTemplateUsed(response, 'tests/form_page.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_landing.html')

    def test_post_valid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertContains(response, "Thank you for your feedback.")
        self.assertTemplateNotUsed(response, 'tests/form_page.html')
        self.assertTemplateUsed(response, 'tests/form_page_landing.html')

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

        # check the default form_submission is added to the context
        self.assertContains(response, "<li>your_email: bob@example.com</li>")

        # Check that an email was sent
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "The subject")
        self.assertIn("Your message: hello world", mail.outbox[0].body)
        self.assertEqual(mail.outbox[0].to, ['to@email.com'])
        self.assertEqual(mail.outbox[0].from_email, 'from@email.com')

        # Check that form submission was saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        self.assertTrue(FormSubmission.objects.filter(page=form_page, form_data__contains='hello world').exists())

    def test_post_unicode_characters(self):
        self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'こんにちは、世界',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check the email
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Your message: こんにちは、世界", mail.outbox[0].body)

        # Check the form submission
        submission = FormSubmission.objects.get()
        submission_data = json.loads(submission.form_data)
        self.assertEqual(submission_data['your_message'], 'こんにちは、世界')

    def test_post_multiple_values(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': 'on', 'bar': 'on', 'baz': 'on'}
        })

        # Check response
        self.assertContains(response, "Thank you for your feedback.")
        self.assertTemplateNotUsed(response, 'tests/form_page.html')
        self.assertTemplateUsed(response, 'tests/form_page_landing.html')

        # Check that the three checkbox values were saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        submission = FormSubmission.objects.filter(
            page=form_page, form_data__contains='hello world'
        )
        self.assertIn("foo", submission[0].form_data)
        self.assertIn("bar", submission[0].form_data)
        self.assertIn("baz", submission[0].form_data)

        # Check that the all the multiple checkbox values are serialised in the
        # email correctly
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("bar", mail.outbox[0].body)
        self.assertIn("foo", mail.outbox[0].body)
        self.assertIn("baz", mail.outbox[0].body)

    def test_post_blank_checkbox(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {},
        })

        # Check response
        self.assertContains(response, "Thank you for your feedback.")
        self.assertTemplateNotUsed(response, 'tests/form_page.html')
        self.assertTemplateUsed(response, 'tests/form_page_landing.html')

        # Check that the checkbox was serialised in the email correctly
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Your choices: ", mail.outbox[0].body)


class TestFormWithCustomSubmission(TestCase, WagtailTestUtils):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page_with_custom_submission()

        self.user = self.login()

    def test_get_form(self):
        response = self.client.get('/contact-us/')

        # Check response
        self.assertContains(response, """<label for="id_your_email">Your email</label>""")
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission_landing.html')
        self.assertNotContains(response, '<div>You must log in first.</div>', html=True)
        self.assertContains(response, '<p>Boring intro text</p>', html=True)

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

    def test_get_form_with_anonymous_user(self):
        self.client.logout()

        response = self.client.get('/contact-us/')

        # Check response
        self.assertNotContains(response, """<label for="id_your_email">Your email</label>""")
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission_landing.html')
        self.assertContains(response, '<div>You must log in first.</div>', html=True)
        self.assertNotContains(response, '<p>Boring intro text</p>', html=True)

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

    def test_post_invalid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob',
            'your_message': 'hello world',
            'your_choices': ''
        })

        # Check response
        self.assertContains(response, "Enter a valid email address.")
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission_landing.html')

    def test_post_valid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertContains(response, "Thank you for your patience!")
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission_landing.html')

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

        # check that the custom form_submission is added to the context
        self.assertContains(response, "<p>User email: test@email.com</p>")

        # Check that an email was sent
        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "The subject")
        self.assertIn("Your message: hello world", mail.outbox[0].body)
        self.assertEqual(mail.outbox[0].to, ['to@email.com'])
        self.assertEqual(mail.outbox[0].from_email, 'from@email.com')

        # Check that form submission was saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        self.assertTrue(CustomFormPageSubmission.objects.filter(page=form_page, form_data__contains='hello world').exists())

    def test_post_form_twice(self):
        # First submission
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission_landing.html')
        self.assertContains(response, '<p>Thank you for your patience!</p>', html=True)
        self.assertNotContains(response, '<div>The form is already filled.</div>', html=True)

        # Check that first form submission was saved correctly
        submissions_qs = CustomFormPageSubmission.objects.filter(user=self.user, page=self.form_page)
        self.assertEqual(submissions_qs.count(), 1)
        self.assertTrue(submissions_qs.filter(form_data__contains='hello world').exists())

        # Second submission
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission_landing.html')
        self.assertNotContains(response, '<p>Thank you for your patience!</p>', html=True)
        self.assertContains(response, '<div>The form is already filled.</div>', html=True)
        self.assertNotContains(response, '<div>You must log in first.</div>', html=True)
        self.assertNotContains(response, '<p>Boring intro text</p>', html=True)

        # Check that first submission exists and second submission wasn't saved
        submissions_qs = CustomFormPageSubmission.objects.filter(user=self.user, page=self.form_page)
        self.assertEqual(submissions_qs.count(), 1)
        self.assertTrue(submissions_qs.filter(form_data__contains='hello world').exists())
        self.assertFalse(submissions_qs.filter(form_data__contains='hello cruel world').exists())

    def test_post_unicode_characters(self):
        self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'こんにちは、世界',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check the email
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Your message: こんにちは、世界", mail.outbox[0].body)

        # Check the form submission
        submission = CustomFormPageSubmission.objects.get()
        submission_data = json.loads(submission.form_data)
        self.assertEqual(submission_data['your_message'], 'こんにちは、世界')

    def test_post_multiple_values(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': 'on', 'bar': 'on', 'baz': 'on'}
        })

        # Check response
        self.assertContains(response, "Thank you for your patience!")
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission_landing.html')

        # Check that the three checkbox values were saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        submission = CustomFormPageSubmission.objects.filter(
            page=form_page, form_data__contains='hello world'
        )
        self.assertIn("foo", submission[0].form_data)
        self.assertIn("bar", submission[0].form_data)
        self.assertIn("baz", submission[0].form_data)

    def test_post_blank_checkbox(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {},
        })

        # Check response
        self.assertContains(response, "Thank you for your patience!")
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission_landing.html')

        # Check that the checkbox was serialised in the email correctly
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Your choices: None", mail.outbox[0].body)


class TestFormSubmissionWithMultipleRecipients(TestCase):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page(to_address='to@email.com, another@email.com')

    def test_post_valid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertContains(response, "Thank you for your feedback.")
        self.assertTemplateNotUsed(response, 'tests/form_page.html')
        self.assertTemplateUsed(response, 'tests/form_page_landing.html')

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

        # Check that one email was sent, but to two recipients
        self.assertEqual(len(mail.outbox), 1)

        self.assertEqual(mail.outbox[0].subject, "The subject")
        self.assertIn("Your message: hello world", mail.outbox[0].body)
        self.assertEqual(mail.outbox[0].from_email, 'from@email.com')
        self.assertEqual(set(mail.outbox[0].to), {'to@email.com', 'another@email.com'})

        # Check that form submission was saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        self.assertTrue(FormSubmission.objects.filter(page=form_page, form_data__contains='hello world').exists())


class TestFormSubmissionWithMultipleRecipientsAndWithCustomSubmission(TestCase, WagtailTestUtils):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page_with_custom_submission(
            to_address='to@email.com, another@email.com'
        )

        self.user = self.login()

    def test_post_valid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertContains(response, "Thank you for your patience!")
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_submission.html')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_submission_landing.html')

        # check that variables defined in get_context are passed through to the template (#1429)
        self.assertContains(response, "<p>hello world</p>")

        # Check that one email was sent, but to two recipients
        self.assertEqual(len(mail.outbox), 1)

        self.assertEqual(mail.outbox[0].subject, "The subject")
        self.assertIn("Your message: hello world", mail.outbox[0].body)
        self.assertEqual(mail.outbox[0].from_email, 'from@email.com')
        self.assertEqual(set(mail.outbox[0].to), {'to@email.com', 'another@email.com'})

        # Check that form submission was saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        self.assertTrue(
            CustomFormPageSubmission.objects.filter(page=form_page, form_data__contains='hello world').exists()
        )


class TestFormWithRedirect(TestCase):
    def setUp(self):
        # Create a form page
        self.form_page = make_form_page_with_redirect(to_address='to@email.com, another@email.com')

    def test_post_valid_form(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''}
        })

        # Check response
        self.assertRedirects(response, '/')

        # Check that one email was sent, but to two recipients
        self.assertEqual(len(mail.outbox), 1)

        self.assertEqual(mail.outbox[0].subject, "The subject")
        self.assertIn("Your message: hello world", mail.outbox[0].body)
        self.assertEqual(mail.outbox[0].from_email, 'from@email.com')
        self.assertEqual(set(mail.outbox[0].to), {'to@email.com', 'another@email.com'})

        # Check that form submission was saved correctly
        form_page = Page.objects.get(url_path='/home/contact-us/')
        self.assertTrue(FormSubmission.objects.filter(page=form_page, form_data__contains='hello world').exists())


class TestFormPageWithCustomFormBuilder(TestCase, WagtailTestUtils):

    def setUp(self):

        home_page = Page.objects.get(url_path='/home/')
        form_page = home_page.add_child(
            instance=FormPageWithCustomFormBuilder(
                title='Support Request',
                slug='support-request',
                to_address='it@jenkins.com',
                from_address='support@jenkins.com',
                subject='Support Request Submitted',
            )
        )
        ExtendedFormField.objects.create(
            page=form_page,
            sort_order=1,
            label='Name',
            field_type='singleline',  # singleline field will be max_length 120
            required=True,
        )
        ExtendedFormField.objects.create(
            page=form_page,
            sort_order=1,
            label='Device IP Address',
            field_type='ipaddress',
            required=True,
        )

    def test_get_form(self):
        response = self.client.get('/support-request/')

        # Check response
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_form_builder.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_form_builder_landing.html')
        self.assertContains(response, '<title>Support Request</title>', html=True)
        # check that max_length attribute has been passed into form
        self.assertContains(response, '<input type="text" name="name" required maxlength="120" id="id_name" />', html=True)
        # check ip address field has rendered
        self.assertContains(response, '<input type="text" name="device_ip_address" required id="id_device_ip_address" />', html=True)

    def test_post_invalid_form(self):
        response = self.client.post('/support-request/', {
            'name': 'very long name longer than 120 characters' * 3,  # invalid
            'device_ip_address': '192.0.2.30',  # valid
        })
        # Check response with invalid character count
        self.assertContains(response, 'Ensure this value has at most 120 characters (it has 123)')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_form_builder.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_form_builder_landing.html')

        response = self.client.post('/support-request/', {
            'name': 'Ron Johnson',  # valid
            'device_ip_address': '3300.192.0.2.30',  # invalid
        })
        # Check response with invalid character count
        self.assertContains(response, 'Enter a valid IPv4 or IPv6 address.')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_form_builder.html')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_form_builder_landing.html')

    def test_post_valid_form(self):
        response = self.client.post('/support-request/', {
            'name': 'Ron Johnson',
            'device_ip_address': '192.0.2.30',
        })

        # Check response
        self.assertContains(response, 'Thank you for submitting a Support Request.')
        self.assertContains(response, 'Ron Johnson')
        self.assertContains(response, '192.0.2.30')
        self.assertTemplateNotUsed(response, 'tests/form_page_with_custom_form_builder.html')
        self.assertTemplateUsed(response, 'tests/form_page_with_custom_form_builder_landing.html')


class TestCleanedDataEmails(TestCase):
    def setUp(self):
        # Create a form page
        self.form_page = make_types_test_form_page()

    def test_empty_field_presence(self):
        self.client.post('/contact-us/', {})

        # Check the email
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Single line text: ", mail.outbox[0].body)
        self.assertIn("Multiline: ", mail.outbox[0].body)
        self.assertIn("Email: ", mail.outbox[0].body)
        self.assertIn("Number: ", mail.outbox[0].body)
        self.assertIn("URL: ", mail.outbox[0].body)
        self.assertIn("Checkbox: ", mail.outbox[0].body)
        self.assertIn("Checkboxes: ", mail.outbox[0].body)
        self.assertIn("Drop down: ", mail.outbox[0].body)
        self.assertIn("Multiple select: ", mail.outbox[0].body)
        self.assertIn("Radio buttons: ", mail.outbox[0].body)
        self.assertIn("Date: ", mail.outbox[0].body)
        self.assertIn("Datetime: ", mail.outbox[0].body)

    def test_email_field_order(self):
        self.client.post('/contact-us/', {})

        line_beginnings = [
            "Single line text: ",
            "Multiline: ",
            "Email: ",
            "Number: ",
            "URL: ",
            "Checkbox: ",
            "Checkboxes: ",
            "Drop down: ",
            "Multiple select: ",
            "Radio buttons: ",
            "Date: ",
            "Datetime: ",
        ]

        # Check the email
        self.assertEqual(len(mail.outbox), 1)
        email_lines = mail.outbox[0].body.split('\n')

        for beginning in line_beginnings:
            message_line = email_lines.pop(0)
            self.assertTrue(message_line.startswith(beginning))

    @override_settings(SHORT_DATE_FORMAT='m/d/Y')
    def test_date_normalization(self):
        self.client.post('/contact-us/', {
            'date': '12/31/17',
        })

        # Check the email
        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Date: 12/31/2017", mail.outbox[0].body)

        self.client.post('/contact-us/', {
            'date': '12/31/1917',
        })

        # Check the email
        self.assertEqual(len(mail.outbox), 2)
        self.assertIn("Date: 12/31/1917", mail.outbox[1].body)

    @override_settings(SHORT_DATETIME_FORMAT='m/d/Y P')
    def test_datetime_normalization(self):
        self.client.post('/contact-us/', {
            'datetime': '12/31/17 4:00:00',
        })

        self.assertEqual(len(mail.outbox), 1)
        self.assertIn("Datetime: 12/31/2017 4 a.m.", mail.outbox[0].body)

        self.client.post('/contact-us/', {
            'datetime': '12/31/1917 21:19',
        })

        self.assertEqual(len(mail.outbox), 2)
        self.assertIn("Datetime: 12/31/1917 9:19 p.m.", mail.outbox[1].body)

        self.client.post('/contact-us/', {
            'datetime': '1910-12-21 21:19:12',
        })

        self.assertEqual(len(mail.outbox), 3)
        self.assertIn("Datetime: 12/21/1910 9:19 p.m.", mail.outbox[2].body)


class TestIssue798(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.login(username='siteeditor', password='password')
        self.form_page = Page.objects.get(url_path='/home/contact-us/').specific

        # Add a number field to the page
        FormField.objects.create(
            page=self.form_page,
            label="Your favourite number",
            field_type='number',
        )

    def test_post(self):
        response = self.client.post('/contact-us/', {
            'your_email': 'bob@example.com',
            'your_message': 'hello world',
            'your_choices': {'foo': '', 'bar': '', 'baz': ''},
            'your_favourite_number': '7.3',
        })

        # Check response
        self.assertTemplateUsed(response, 'tests/form_page_landing.html')

        # Check that form submission was saved correctly
        self.assertTrue(FormSubmission.objects.filter(page=self.form_page, form_data__contains='hello world').exists())
        self.assertTrue(FormSubmission.objects.filter(page=self.form_page, form_data__contains='7.3').exists())


class TestNonHtmlExtension(TestCase):
    fixtures = ['test.json']

    def test_non_html_extension(self):
        form_page = JadeFormPage(title="test")
        self.assertEqual(form_page.landing_page_template, "tests/form_page_landing.jade")


class TestLegacyFormFieldCleanNameChecks(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.login(username='siteeditor', password='password')
        self.form_page = Page.objects.get(url_path='/home/contact-us-one-more-time/').specific

    def test_form_field_clean_name_update_on_checks(self):
        fields_before_checks = [
            (field.label, field.clean_name,)
            for field in FormFieldWithCustomSubmission.objects.all()
        ]

        self.assertEqual(fields_before_checks, [
            ('Your email', ''),
            ('Your message', ''),
            ('Your choices', ''),
        ])

        # running checks should show an info message AND update blank clean_name values

        messages = FormFieldWithCustomSubmission.check()

        self.assertEqual(
            messages,
            [Info('Added `clean_name` on 3 form field(s)', obj=FormFieldWithCustomSubmission)]
        )

        fields_after_checks = [
            (field.label, field.clean_name,)
            for field in FormFieldWithCustomSubmission.objects.all()
        ]

        self.assertEqual(fields_after_checks, [
            ('Your email', 'your-email'),  # kebab case, legacy format
            ('Your message', 'your-message'),
            ('Your choices', 'your-choices'),
        ])

        # running checks again should return no messages as fields no longer need changing
        self.assertEqual(FormFieldWithCustomSubmission.check(), [])

        # creating a new field should use the non-legacy clean_name format

        field = FormFieldWithCustomSubmission.objects.create(
            page=self.form_page,
            label="Your FAVOURITE #number",
            field_type='number',
        )

        self.assertEqual(field.clean_name, 'your_favourite_number')



from .assign_role import AssignRoleBulkAction
from .delete import DeleteBulkAction
from .set_active_state import SetActiveStateBulkAction


__all__ = ['AssignRoleBulkAction', 'DeleteBulkAction', 'SetActiveStateBulkAction']



from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.db import models

from wagtail.admin.auth import permission_denied  # noqa
from wagtail.edit_handlers import FieldPanel

from .fields import ConvertedValueField


# make sure we can import wagtail.admin.auth here without triggering a circular import
# (which is easily done because it's dealing with django.contrib.auth views which depend
# on the user model)


class CustomUserManager(BaseUserManager):
    def _create_user(self, username, email, password,
                     is_staff, is_superuser, is_active=True, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        """
        if not username:
            raise ValueError('The given username must be set')
        email = self.normalize_email(email)
        user = self.model(username=username, email=email,
                          is_staff=is_staff, is_active=is_active,
                          is_superuser=is_superuser, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, username, email=None, password=None, **extra_fields):
        return self._create_user(username, email, password, False, False,
                                 **extra_fields)

    def create_superuser(self, username, email, password, **extra_fields):
        return self._create_user(username, email, password, True, True,
                                 **extra_fields)


class CustomUser(AbstractBaseUser, PermissionsMixin):
    identifier = ConvertedValueField(primary_key=True)
    username = models.CharField(max_length=100, unique=True)
    email = models.EmailField(max_length=255, blank=True)
    is_staff = models.BooleanField(default=True)
    is_active = models.BooleanField(default=True)
    first_name = models.CharField(max_length=50, blank=True)
    last_name = models.CharField(max_length=50, blank=True)
    country = models.CharField(max_length=100, blank=True)
    attachment = models.FileField(blank=True)

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    objects = CustomUserManager()

    def get_full_name(self):
        return self.first_name + ' ' + self.last_name

    def get_short_name(self):
        return self.first_name

    panels = [
        FieldPanel('first_name'),
        FieldPanel('last_name'),
    ]



from django.contrib import messages
from django.contrib.admin.utils import unquote
from django.core.exceptions import PermissionDenied
from django.db import transaction
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse
from django.utils.translation import gettext as _
from django.utils.translation import gettext_lazy
from django.views.generic import TemplateView
from django.views.generic.detail import SingleObjectMixin

from wagtail.contrib.snippets.views.snippets import get_snippet_model_from_url_params
from wagtail.models import Page, TranslatableMixin

from .forms import SubmitTranslationForm


class SubmitTranslationView(SingleObjectMixin, TemplateView):
    template_name = "simple_translation/admin/submit_translation.html"
    title = gettext_lazy("Translate")

    def get_title(self):
        return self.title

    def get_subtitle(self):
        return str(self.object)

    def get_form(self):
        if self.request.method == "POST":
            return SubmitTranslationForm(self.object, self.request.POST)
        return SubmitTranslationForm(self.object)

    def get_success_url(self):
        raise NotImplementedError

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            {
                "form": self.get_form(),
            }
        )
        return context

    def post(self, request, **kwargs):  # pragma: no mccabe
        form = self.get_form()

        if form.is_valid():
            with transaction.atomic():
                for locale in form.cleaned_data["locales"]:
                    if isinstance(self.object, Page):
                        self.object.copy_for_translation(locale)
                        if form.cleaned_data["include_subtree"]:

                            def _walk(current_page):
                                for child_page in current_page.get_children():
                                    child_page.copy_for_translation(locale)

                                    if child_page.numchild:
                                        _walk(child_page)

                            _walk(self.object)
                    else:
                        self.object.copy_for_translation(
                            locale
                        ).save()  # pragma: no cover

                if len(form.cleaned_data["locales"]) == 1:
                    locales = form.cleaned_data["locales"][0].get_display_name()
                else:
                    # Note: always plural
                    locales = _("{} locales").format(len(form.cleaned_data["locales"]))

                messages.success(self.request, self.get_success_message(locales))

                return redirect(self.get_success_url())

        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)

    def dispatch(self, request, *args, **kwargs):
        if not request.user.has_perms(["simple_translation.submit_translation"]):
            raise PermissionDenied

        self.object = self.get_object()
        return super().dispatch(request, *args, **kwargs)


class SubmitPageTranslationView(SubmitTranslationView):
    title = gettext_lazy("Translate page")

    def get_subtitle(self):
        return self.object.get_admin_display_title()

    def get_object(self):
        page = get_object_or_404(Page, id=self.kwargs["page_id"]).specific

        # Can't translate the root page
        if page.is_root():
            raise Http404

        return page

    def get_success_url(self):
        return reverse("wagtailadmin_explore", args=[self.get_object().get_parent().id])

    def get_success_message(self, locales):
        return _(
            "The page '{page_title}' was successfully created in {locales}"
        ).format(page_title=self.object.get_admin_display_title(), locales=locales)


class SubmitSnippetTranslationView(SubmitTranslationView):
    def get_title(self):
        return _("Translate {model_name}").format(
            model_name=self.object._meta.verbose_name
        )

    def get_object(self):
        model = get_snippet_model_from_url_params(
            self.kwargs["app_label"], self.kwargs["model_name"]
        )

        if not issubclass(model, TranslatableMixin):
            raise Http404

        return get_object_or_404(model, pk=unquote(self.kwargs["pk"]))

    def get_success_url(self):
        return reverse(
            "wagtailsnippets:edit",
            args=[
                self.kwargs["app_label"],
                self.kwargs["model_name"],
                self.kwargs["pk"],
            ],
        )

    def get_success_message(self, locales):
        return _("Successfully created {locales} for {model_name} '{object}'").format(
            model_name=self.object._meta.verbose_name,
            object=str(self.object),
            locales=locales,
        )



# Generated by Django 3.1.2 on 2020-10-15 00:52

from django.db import migrations


def add_choose_permission_to_admin_groups(apps, _schema_editor):
    ContentType = apps.get_model('contenttypes.ContentType')
    Permission = apps.get_model('auth.Permission')
    Group = apps.get_model('auth.Group')

    # Get document content type
    document_content_type, _created = ContentType.objects.get_or_create(
        model='document',
        app_label='wagtaildocs'
    )

    # Create the Choose permission (if it doesn't already exist)
    choose_document_permission, _created = Permission.objects.get_or_create(
        content_type=document_content_type,
        codename='choose_document',
        defaults={'name': 'Can choose document'}
    )

    # Assign it to all groups which have "Access the Wagtail admin" permission.
    # This emulates the previous behavior, where everyone who would access the admin
    # could choose any document in any Collection, because choosing wasn't permissioned.
    for group in Group.objects.filter(permissions__codename='access_admin'):
        group.permissions.add(choose_document_permission)


def remove_choose_permission(apps, _schema_editor):
    """Reverse the above additions of permissions."""
    ContentType = apps.get_model('contenttypes.ContentType')
    Permission = apps.get_model('auth.Permission')
    document_content_type = ContentType.objects.get(
        model='document',
        app_label='wagtaildocs',
    )
    # This cascades to Group
    Permission.objects.filter(
        content_type=document_content_type,
        codename='choose_document'
    ).delete()


def get_choose_permission(apps):
    Permission = apps.get_model('auth.Permission')
    ContentType = apps.get_model('contenttypes.ContentType')

    document_content_type, _created = ContentType.objects.get_or_create(
        model='document',
        app_label='wagtaildocs',
    )
    return Permission.objects.filter(
        content_type=document_content_type,
        codename__in=['choose_document']
    ).first()


def copy_choose_permission_to_collections(apps, _schema_editor):
    Collection = apps.get_model('wagtailcore.Collection')
    Group = apps.get_model('auth.Group')
    GroupCollectionPermission = apps.get_model('wagtailcore.GroupCollectionPermission')

    root_collection = Collection.objects.get(depth=1)

    permission = get_choose_permission(apps)
    if permission:
        for group in Group.objects.filter(permissions=permission):
            GroupCollectionPermission.objects.create(
                group=group,
                collection=root_collection,
                permission=permission
            )


def remove_choose_permission_from_collections(apps, _schema_editor):
    GroupCollectionPermission = apps.get_model('wagtailcore.GroupCollectionPermission')
    choose_permission = get_choose_permission(apps)
    if choose_permission:
        GroupCollectionPermission.objects.filter(permission=choose_permission).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('wagtaildocs', '0010_document_file_hash'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'permissions': [('choose_document', 'Can choose document')], 'verbose_name': 'document', 'verbose_name_plural': 'documents'},
        ),
        migrations.RunPython(add_choose_permission_to_admin_groups, remove_choose_permission),
        migrations.RunPython(copy_choose_permission_to_collections, remove_choose_permission_from_collections),
    ]



from django.core.exceptions import PermissionDenied
from django.db import transaction
from django.shortcuts import get_object_or_404, redirect
from django.template.response import TemplateResponse
from django.utils.translation import gettext as _

from wagtail import hooks
from wagtail.admin import messages
from wagtail.admin.views.pages.utils import get_valid_next_url_from_request
from wagtail.models import Page, logging


def convert_alias(request, page_id):
    page = get_object_or_404(Page, id=page_id, alias_of_id__isnull=False).specific
    if not page.permissions_for_user(request.user).can_edit():
        raise PermissionDenied

    with transaction.atomic():
        for fn in hooks.get_hooks('before_convert_alias_page'):
            result = fn(request, page)
            if hasattr(result, 'status_code'):
                return result

        next_url = get_valid_next_url_from_request(request)

        if request.method == 'POST':
            page.alias_of_id = None
            page.save(update_fields=['alias_of_id'], clean=False)

            # Create an initial revision
            revision = page.save_revision(user=request.user, changed=False, clean=False)

            if page.live:
                page.live_revision = revision
                page.save(update_fields=['live_revision'], clean=False)

            # Log
            logging.PageLogEntry.objects.log_action(
                instance=page,
                revision=revision,
                action='wagtail.convert_alias',
                user=request.user,
                data={
                    'page': {
                        'id': page.id,
                        'title': page.get_admin_display_title()
                    },
                },
            )

            messages.success(request, _("Page '{0}' has been converted into an ordinary page.").format(page.get_admin_display_title()))

            for fn in hooks.get_hooks('after_convert_alias_page'):
                result = fn(request, page)
                if hasattr(result, 'status_code'):
                    return result

            if next_url:
                return redirect(next_url)
            return redirect('wagtailadmin_pages:edit', page.id)

    return TemplateResponse(request, 'wagtailadmin/pages/confirm_convert_alias.html', {
        'page': page,
        'next': next_url,
    })



from django.utils.translation import gettext_lazy as _
from django.utils.translation import ngettext

from wagtail.admin.views.pages.bulk_actions.page_bulk_action import PageBulkAction


class UnpublishBulkAction(PageBulkAction):
    display_name = _("Unpublish")
    action_type = "unpublish"
    aria_label = _("Unpublish selected pages")
    template_name = "wagtailadmin/pages/bulk_actions/confirm_bulk_unpublish.html"
    action_priority = 50

    def check_perm(self, page):
        return page.permissions_for_user(self.request.user).can_unpublish()

    def object_context(self, page):
        return {
            **super().object_context(page),
            'live_descendant_count': page.get_descendants().live().count(),
        }

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['has_live_descendants'] = any(map(lambda x: x['live_descendant_count'] > 0, context['items']))
        return context

    def get_execution_context(self):
        return {
            **super().get_execution_context(),
            'permission_checker': self.check_perm,
            'include_descendants': self.cleaned_form.cleaned_data['include_descendants'],
        }

    @classmethod
    def execute_action(cls, objects, include_descendants=False, user=None, permission_checker=None, **kwargs):
        num_parent_objects, num_child_objects = 0, 0
        for page in objects:
            page.unpublish(user=user)
            num_parent_objects += 1

            if include_descendants:
                for live_descendant_page in page.get_descendants().live().defer_streamfields().specific():
                    if user is None or permission_checker(live_descendant_page):
                        live_descendant_page.unpublish()
                        num_child_objects += 1
        return num_parent_objects, num_child_objects

    def get_success_message(self, num_parent_objects, num_child_objects):
        include_descendants = self.cleaned_form.cleaned_data['include_descendants']
        if num_parent_objects == 1:
            if include_descendants:
                if num_child_objects == 0:
                    success_message = _("1 page has been unpublished")
                else:
                    success_message = ngettext(
                        "1 page and %(num_child_objects)d child page have been unpublished",
                        "1 page and %(num_child_objects)d child pages have been unpublished",
                        num_child_objects
                    ) % {
                        'num_child_objects': num_child_objects
                    }
            else:
                success_message = _("1 page has been unpublished")
        else:
            if include_descendants:
                if num_child_objects == 0:
                    success_message = _("%(num_parent_objects)d pages have been unpublished") % {'num_parent_objects': num_parent_objects}
                else:
                    success_message = ngettext(
                        "%(num_parent_objects)d pages and %(num_child_objects)d child page have been unpublished",
                        "%(num_parent_objects)d pages and %(num_child_objects)d child pages have been unpublished",
                        num_child_objects
                    ) % {
                        'num_child_objects': num_child_objects,
                        'num_parent_objects': num_parent_objects
                    }
            else:
                success_message = _("%(num_parent_objects)d pages have been unpublished") % {'num_parent_objects': num_parent_objects}
        return success_message



from django import forms
from telepath import Adapter, AdapterRegistry, JSContextBase  # noqa

from wagtail.admin.staticfiles import versioned_static


class WagtailJSContextBase(JSContextBase):
    @property
    def base_media(self):
        return forms.Media(js=[
            versioned_static(self.telepath_js_path),
        ])


class WagtailAdapterRegistry(AdapterRegistry):
    js_context_base_class = WagtailJSContextBase


registry = WagtailAdapterRegistry(telepath_js_path='wagtailadmin/js/telepath/telepath.js')
JSContext = registry.js_context_class


def register(adapter, cls):
    registry.register(adapter, cls)


def adapter(js_constructor, base=Adapter):
    """
    Allows a class to implement its adapting logic with a `js_args()` method on the class itself.
    This just helps reduce the amount of code you have to write.

    For example:

        @adapter('wagtail.mywidget')
        class MyWidget():
            ...

            def js_args(self):
                return [
                    self.foo,
                ]

    Is equivalent to:

        class MyWidget():
            ...


        class MyWidgetAdapter(Adapter):
            js_constructor = 'wagtail.mywidget'

            def js_args(self, obj):
                return [
                    self.foo,
                ]
    """
    def _wrapper(cls):
        ClassAdapter = type(cls.__name__ + 'Adapter', (base, ), {
            'js_constructor': js_constructor,
            'js_args': lambda self, obj: obj.js_args(),
        })

        register(ClassAdapter(), cls)

        return cls

    return _wrapper



# Generated by Django 3.0.5 on 2020-07-09 21:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailcore', '0051_taskstate_comment'),
    ]

    operations = [
        migrations.CreateModel(
            name='PageLogEntry',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.TextField()),
                ('action', models.CharField(blank=True, db_index=True, max_length=255)),
                ('data_json', models.TextField(blank=True)),
                ('timestamp', models.DateTimeField(verbose_name='timestamp (UTC)')),
                ('content_changed', models.BooleanField(db_index=True, default=False)),
                ('deleted', models.BooleanField(default=False)),
                ('content_type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='contenttypes.ContentType', verbose_name='content type')),
                ('page', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='wagtailcore.Page')),
                ('revision', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='wagtailcore.PageRevision')),
                ('user', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'page log entry',
                'verbose_name_plural': 'page log entries',
                'ordering': ['-timestamp', '-id'],
            },
        ),
    ]



from django.urls import path

from wagtail.contrib.search_promotions import views


app_name = 'wagtailsearchpromotions'
urlpatterns = [
    path('', views.index, name='index'),
    path('add/', views.add, name='add'),
    path('<int:query_id>/', views.edit, name='edit'),
    path('<int:query_id>/delete/', views.delete, name='delete'),
]



import sqlite3
import unittest

from unittest import skip

from django.db import connection
from django.test.testcases import TestCase
from django.test.utils import override_settings

from wagtail.search.tests.test_backends import BackendTests


@unittest.skipUnless(connection.vendor == 'sqlite', "The current database is not SQLite")
@unittest.skipIf(sqlite3.sqlite_version_info < (3, 19, 0), "This SQLite version is not supported")
@override_settings(WAGTAILSEARCH_BACKENDS={
    'default': {
        'BACKEND': 'wagtail.search.backends.database.sqlite.sqlite',
    }
})
class TestSQLiteSearchBackend(BackendTests, TestCase):
    backend_path = 'wagtail.search.backends.database.sqlite.sqlite'

    @skip("The SQLite backend doesn't support boosting.")
    def test_search_boosting_on_related_fields(self):
        return super().test_search_boosting_on_related_fields()

    @skip("The SQLite backend doesn't support boosting.")
    def test_boost(self):
        return super().test_boost()

    @skip("The SQLite backend doesn't score annotations.")
    def test_annotate_score(self):
        return super().test_annotate_score()

    @skip("The SQLite backend doesn't score annotations.")
    def test_annotate_score_with_slice(self):
        return super().test_annotate_score_with_slice()

    @skip("The SQLite backend doesn't support autocomplete.")
    def test_autocomplete(self):
        return super().test_autocomplete()

    @skip("The SQLite backend doesn't support autocomplete.")
    def test_autocomplete_not_affected_by_stemming(self):
        return super().test_autocomplete_not_affected_by_stemming()

    @skip("The SQLite backend doesn't support autocomplete.")
    def test_autocomplete_uses_autocompletefield(self):
        return super().test_autocomplete_uses_autocompletefield()

    @skip("The SQLite backend doesn't support autocomplete.")
    def test_autocomplete_with_fields_arg(self):
        return super().test_autocomplete_with_fields_arg()

    @skip("The SQLite backend doesn't guarantee correct ranking of results.")
    def test_ranking(self):
        return super().test_ranking()



import json

from django import forms
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

import wagtail.models.workflows

from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.widgets import AdminChooser


class AdminTaskChooser(AdminChooser):
    choose_one_text = _('Choose a task')
    choose_another_text = _('Choose another task')
    link_to_chosen_text = _('Edit this task')

    def render_html(self, name, value, attrs):
        task, value = self.get_instance_and_id(wagtail.models.workflows.Task, value)
        original_field_html = super().render_html(name, value, attrs)

        return render_to_string("wagtailadmin/workflows/widgets/task_chooser.html", {
            'widget': self,
            'original_field_html': original_field_html,
            'attrs': attrs,
            'value': value,
            'display_title': task.name if task else '',
            'edit_url': reverse('wagtailadmin_workflows:edit_task', args=[task.id]) if task else '',
        })

    def render_js_init(self, id_, name, value):
        return "createTaskChooser({0});".format(json.dumps(id_))

    @property
    def media(self):
        return forms.Media(js=[
            versioned_static('wagtailadmin/js/task-chooser-modal.js'),
            versioned_static('wagtailadmin/js/task-chooser.js'),
        ])



from wagtail.admin.mail import (
    GroupApprovalTaskStateSubmissionEmailNotifier, WorkflowStateApprovalEmailNotifier,
    WorkflowStateRejectionEmailNotifier, WorkflowStateSubmissionEmailNotifier)
from wagtail.models import workflows
from wagtail.signals import task_submitted, workflow_approved, workflow_rejected, workflow_submitted


task_submission_email_notifier = GroupApprovalTaskStateSubmissionEmailNotifier()
workflow_submission_email_notifier = WorkflowStateSubmissionEmailNotifier()
workflow_approval_email_notifier = WorkflowStateApprovalEmailNotifier()
workflow_rejection_email_notifier = WorkflowStateRejectionEmailNotifier()


def register_signal_handlers():
    task_submitted.connect(task_submission_email_notifier, sender=workflows.TaskState, dispatch_uid='group_approval_task_submitted_email_notification')

    workflow_submitted.connect(workflow_submission_email_notifier, sender=workflows.WorkflowState, dispatch_uid='workflow_state_submitted_email_notification')
    workflow_rejected.connect(workflow_rejection_email_notifier, sender=workflows.WorkflowState, dispatch_uid='workflow_state_rejected_email_notification')
    workflow_approved.connect(workflow_approval_email_notifier, sender=workflows.WorkflowState, dispatch_uid='workflow_state_approved_email_notification')



# Generated by Django 2.2.13 on 2020-07-01 14:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0046_site_name_remove_null'),
        ('tests', '0051_tag_verbose_name'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customdocument',
            options={},
        ),
        migrations.AlterUniqueTogether(
            name='customdocument',
            unique_together={('title', 'collection')},
        ),
        migrations.AlterUniqueTogether(
            name='customimage',
            unique_together={('title', 'collection')},
        ),
    ]



from django.contrib.postgres.search import SearchQueryCombinable, SearchQueryField
from django.db.models.expressions import Expression, Value


class LexemeCombinable(Expression):
    BITAND = '&'
    BITOR = '|'

    def _combine(self, other, connector, reversed, node=None):
        if not isinstance(other, LexemeCombinable):
            raise TypeError(
                'Lexeme can only be combined with other Lexemes, '
                'got {}.'.format(type(other))
            )
        if reversed:
            return CombinedLexeme(other, connector, self)
        return CombinedLexeme(self, connector, other)

    # On Combinable, these are not implemented to reduce confusion with Q. In
    # this case we are actually (ab)using them to do logical combination so
    # it's consistent with other usage in Django.
    def bitand(self, other):
        return self._combine(other, self.BITAND, False)

    def bitor(self, other):
        return self._combine(other, self.BITOR, False)

    def __or__(self, other):
        return self._combine(other, self.BITOR, False)

    def __and__(self, other):
        return self._combine(other, self.BITAND, False)


class Lexeme(LexemeCombinable, Value):
    _output_field = SearchQueryField()

    def __init__(self, value, output_field=None, *, invert=False, prefix=False, weight=None):
        self.prefix = prefix
        self.invert = invert
        self.weight = weight
        super().__init__(value, output_field=output_field)

    def as_sql(self, compiler, connection):
        param = "'%s'" % self.value.replace("'", "''").replace("\\", "\\\\")

        template = "%s"

        label = ''
        if self.prefix:
            label += '*'
        if self.weight:
            label += self.weight

        if label:
            param = '{}:{}'.format(param, label)
        if self.invert:
            param = '!{}'.format(param)

        return template, [param]


class CombinedLexeme(LexemeCombinable):
    _output_field = SearchQueryField()

    def __init__(self, lhs, connector, rhs, output_field=None):
        super().__init__(output_field=output_field)
        self.connector = connector
        self.lhs = lhs
        self.rhs = rhs

    def as_sql(self, compiler, connection):
        value_params = []
        lsql, params = compiler.compile(self.lhs)
        value_params.extend(params)

        rsql, params = compiler.compile(self.rhs)
        value_params.extend(params)

        combined_sql = '({} {} {})'.format(lsql, self.connector, rsql)
        combined_value = combined_sql % tuple(value_params)
        return '%s', [combined_value]


# This class is required for Django 3.0 support and below
# In Django 3.1 onwards, we can replace this with SearchQuery(expression, search_type='raw')
# The PR for the functionality we need is here: https://github.com/django/django/pull/12525
class RawSearchQuery(SearchQueryCombinable, Expression):
    _output_field = SearchQueryField()

    def __init__(self, expressions, output_field=None, *, config=None, invert=False):
        self.config = config
        self.invert = invert
        self.expressions = expressions
        super().__init__(output_field=output_field)

    def resolve_expression(self, query=None, allow_joins=True, reuse=None, summarize=False, for_save=False):
        resolved = super().resolve_expression(query, allow_joins, reuse, summarize, for_save)
        if self.config:
            if not hasattr(self.config, 'resolve_expression'):
                resolved.config = Value(self.config).resolve_expression(query, allow_joins, reuse, summarize, for_save)
            else:
                resolved.config = self.config.resolve_expression(query, allow_joins, reuse, summarize, for_save)
        return resolved

    def as_sql(self, compiler, connection):
        sql, params, = compiler.compile(self.expressions)

        if self.config:
            config_sql, config_params = compiler.compile(self.config)
            template = 'to_tsquery({}::regconfig, {})'.format(config_sql, sql)
            params = config_params + params
        else:
            template = 'to_tsquery({})'.format(sql)
        if self.invert:
            template = '!!({})'.format(template)
        return template, params

    def _combine(self, *args, **kwargs):
        combined = super()._combine(*args, **kwargs)
        combined.output_field = SearchQueryField()
        return combined

    def __invert__(self):
        return type(self)(self.lexeme, config=self.config, invert=not self.invert)



import sys

from wagtail.utils.deprecation import MovedDefinitionHandler, RemovedInWagtail217Warning


MOVED_DEFINITIONS = {
    'QueryForm': ('wagtail.contrib.search_promotions.forms', 'QueryForm'),
}

sys.modules[__name__] = MovedDefinitionHandler(sys.modules[__name__], MOVED_DEFINITIONS, RemovedInWagtail217Warning)



# Generated by Django 3.0.9 on 2020-08-04 12:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0057_page_locale_fields_notnull'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='alias_of',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='aliases', to='wagtailcore.Page'),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WagtailAppConfig(AppConfig):
    name = 'wagtail'
    label = 'wagtailcore'
    verbose_name = _("Wagtail core")
    default_auto_field = 'django.db.models.AutoField'
    group_viewset = 'wagtail.admin.views.groups.GroupViewSet'

    def ready(self):
        from wagtail.admin.signal_handlers import \
            register_signal_handlers as register_admin_signal_handlers
        from wagtail.signal_handlers import register_signal_handlers

        register_signal_handlers()
        register_admin_signal_handlers()

        from wagtail import widget_adapters  # noqa



# Generated by Django 3.1.8 on 2021-10-01 14:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailusers', '0011_userprofile_related_name_change'),
        ('wagtailcore', '0068_copy_userprofiles'),
    ]

    operations = [
        migrations.DeleteModel(
            name='UserProfile',
        ),
    ]



# Generated by Django 3.2.5 on 2021-09-01 14:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0064_log_timestamp_indexes'),
    ]

    operations = [
        migrations.AddField(
            model_name='modellogentry',
            name='uuid',
            field=models.UUIDField(blank=True, editable=False, help_text='Log entries that happened as part of the same user action are assigned the same UUID', null=True),
        ),
        migrations.AddField(
            model_name='pagelogentry',
            name='uuid',
            field=models.UUIDField(blank=True, editable=False, help_text='Log entries that happened as part of the same user action are assigned the same UUID', null=True),
        ),
    ]



# -*- coding: utf-8 -*
from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
from django.test import TestCase, override_settings
from django.utils.text import slugify
from django.utils.translation import _trans
from django.utils.translation import gettext_lazy as _

from wagtail.coreutils import (
    accepts_kwarg, camelcase_to_underscore, cautious_slugify, find_available_slug,
    get_content_languages, get_supported_content_language_variant, multigetattr, safe_snake_case,
    string_to_ascii)
from wagtail.models import Page


class TestCamelCaseToUnderscore(TestCase):

    def test_camelcase_to_underscore(self):
        test_cases = [
            ('HelloWorld', 'hello_world'),
            ('longValueWithVarious subStrings', 'long_value_with_various sub_strings')
        ]

        for (original, expected_result) in test_cases:
            self.assertEqual(camelcase_to_underscore(original), expected_result)


class TestStringToAscii(TestCase):

    def test_string_to_ascii(self):
        test_cases = [
            (u'30 \U0001d5c4\U0001d5c6/\U0001d5c1', '30 km/h'),
            (u'\u5317\u4EB0', 'BeiJing'),
            ('ぁ あ ぃ い ぅ う ぇ', 'a a i i u u e'),
            ('Ա Բ Գ Դ Ե Զ Է Ը Թ Ժ Ի Լ Խ Ծ Կ Հ Ձ Ղ Ճ Մ Յ Ն', 'A B G D E Z E Y T\' Zh I L Kh Ts K H Dz Gh Ch M Y N'),
            ('Спорт!', 'Sport!'),
            ('Straßenbahn', 'Strassenbahn'),
            ('Hello world', 'Hello world'),
            ('Ā ā Ă ă Ą ą Ć ć Ĉ ĉ Ċ ċ Č č Ď ď Đ', 'A a A a A a C c C c C c C c D d D'),
            ('〔山脈〕', '[ShanMai]'),
        ]

        for (original, expected_result) in test_cases:
            self.assertEqual(string_to_ascii(original), expected_result)


class TestCautiousSlugify(TestCase):

    def test_behaves_same_as_slugify_for_latin_chars(self):
        test_cases = [
            ('', ''),
            ('???', ''),
            ('Hello world', 'hello-world'),
            ('Hello_world', 'hello_world'),
            ('Hellö wörld', 'hello-world'),
            ('Hello   world', 'hello-world'),
            ('   Hello world   ', 'hello-world'),
            ('Hello, world!', 'hello-world'),
            ('Hello*world', 'helloworld'),
            ('Hello☃world', 'helloworld'),
        ]

        for (original, expected_result) in test_cases:
            self.assertEqual(slugify(original), expected_result)
            self.assertEqual(cautious_slugify(original), expected_result)

    def test_escapes_non_latin_chars(self):
        test_cases = [
            ('Straßenbahn', 'straxdfenbahn'),
            ('Спорт!', 'u0421u043fu043eu0440u0442'),
            ('〔山脈〕', 'u5c71u8108'),
        ]

        for (original, expected_result) in test_cases:
            self.assertEqual(cautious_slugify(original), expected_result)


class TestSafeSnakeCase(TestCase):

    def test_strings_with_latin_chars(self):
        test_cases = [
            ('', ''),
            ('???', ''),
            ('using-Hyphen', 'using_hyphen'),
            ('en–⁠dash', 'endash'),  # unicode non-letter characters stripped
            ('  em—dash ', 'emdash'),  # unicode non-letter characters stripped
            ('horizontal―BAR', 'horizontalbar'),  # unicode non-letter characters stripped
            ('Hello world', 'hello_world'),
            ('Hello_world', 'hello_world'),
            ('Hellö wörld', 'hello_world'),
            ('Hello   world', 'hello_world'),
            ('   Hello world   ', 'hello_world'),
            ('Hello, world!', 'hello_world'),
            ('Hello*world', 'helloworld'),
            ('Screenshot_2020-05-29 Screenshot(1).png', 'screenshot_2020_05_29_screenshot1png')
        ]

        for (original, expected_result) in test_cases:
            self.assertEqual(safe_snake_case(original), expected_result)

    def test_strings_with__non_latin_chars(self):
        test_cases = [
            ('Straßenbahn Straßenbahn', 'straxdfenbahn_straxdfenbahn'),
            ('Сп орт!', 'u0421u043f_u043eu0440u0442'),
        ]

        for (original, expected_result) in test_cases:
            self.assertEqual(safe_snake_case(original), expected_result)


class TestAcceptsKwarg(TestCase):
    def test_accepts_kwarg(self):
        def func_without_banana(apple, orange=42):
            pass

        def func_with_banana(apple, banana=42):
            pass

        def func_with_kwargs(apple, **kwargs):
            pass

        self.assertFalse(accepts_kwarg(func_without_banana, 'banana'))
        self.assertTrue(accepts_kwarg(func_with_banana, 'banana'))
        self.assertTrue(accepts_kwarg(func_with_kwargs, 'banana'))


class TestFindAvailableSlug(TestCase):
    def setUp(self):
        self.root_page = Page.objects.get(depth=1)
        self.home_page = Page.objects.get(depth=2)

        self.second_home_page = self.root_page.add_child(
            instance=Page(title="Second homepage", slug="home-1")
        )

    def test_find_available_slug(self):
        with self.assertNumQueries(1):
            slug = find_available_slug(self.root_page, "unique-slug")

        self.assertEqual(slug, "unique-slug")

    def test_find_available_slug_already_used(self):
        # Even though the first two slugs are already used, this still requires only one query to find a unique one
        with self.assertNumQueries(1):
            slug = find_available_slug(self.root_page, "home")

        self.assertEqual(slug, "home-2")

    def test_find_available_slug_ignore_page_id(self):
        with self.assertNumQueries(1):
            slug = find_available_slug(self.root_page, "home", ignore_page_id=self.second_home_page.id)

        self.assertEqual(slug, "home-1")


@override_settings(
    USE_I18N=True,
    WAGTAIL_I18N_ENABLED=True,
    LANGUAGES=[
        ('en', 'English'),
        ('de', 'German'),
        ('de-at', 'Austrian German'),
        ('pt-br', 'Portuguese (Brazil)'),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ('en', 'English'),
        ('de', 'German'),
        ('de-at', 'Austrian German'),
        ('pt-br', 'Portuguese (Brazil)'),
    ],
)
class TestGetContentLanguages(TestCase):
    def test_get_content_languages(self):
        self.assertEqual(get_content_languages(), {
            'de': 'German',
            'de-at': 'Austrian German',
            'en': 'English',
            'pt-br': 'Portuguese (Brazil)'
        })

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ('en', 'English'),
            ('de', 'German'),
        ],
    )
    def test_can_be_different_to_django_languages(self):
        self.assertEqual(get_content_languages(), {
            'de': 'German',
            'en': 'English',
        })

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ('en', _('English')),
            ('de', _('German')),
        ],
    )
    def test_can_be_a_translation_proxy(self):
        self.assertEqual(get_content_languages(), {
            'de': 'German',
            'en': 'English',
        })

    @override_settings(
        WAGTAIL_CONTENT_LANGUAGES=[
            ('en', 'English'),
            ('de', 'German'),
            ('zh', 'Chinese'),
        ],
    )
    def test_must_be_subset_of_django_languages(self):
        with self.assertRaises(ImproperlyConfigured) as e:
            get_content_languages()

        self.assertEqual(e.exception.args, ("The language zh is specified in WAGTAIL_CONTENT_LANGUAGES but not LANGUAGES. WAGTAIL_CONTENT_LANGUAGES must be a subset of LANGUAGES.", ))


@override_settings(
    USE_I18N=True,
    WAGTAIL_I18N_ENABLED=True,
    LANGUAGES=[
        ('en', 'English'),
        ('de', 'German'),
        ('de-at', 'Austrian German'),
        ('pt-br', 'Portuguese (Brazil)'),
    ],
    WAGTAIL_CONTENT_LANGUAGES=[
        ('en', 'English'),
        ('de', 'German'),
        ('de-at', 'Austrian German'),
        ('pt-br', 'Portuguese (Brazil)'),
    ],
)
class TestGetSupportedContentLanguageVariant(TestCase):
    # From: https://github.com/django/django/blob/9e57b1efb5205bd94462e9de35254ec5ea6eb04e/tests/i18n/tests.py#L1481
    def test_get_supported_content_language_variant(self):
        g = get_supported_content_language_variant
        self.assertEqual(g('en'), 'en')
        self.assertEqual(g('en-gb'), 'en')
        self.assertEqual(g('de'), 'de')
        self.assertEqual(g('de-at'), 'de-at')
        self.assertEqual(g('de-ch'), 'de')
        self.assertEqual(g('pt-br'), 'pt-br')
        self.assertEqual(g('pt'), 'pt-br')
        self.assertEqual(g('pt-pt'), 'pt-br')
        with self.assertRaises(LookupError):
            g('pt', strict=True)
        with self.assertRaises(LookupError):
            g('pt-pt', strict=True)
        with self.assertRaises(LookupError):
            g('xyz')
        with self.assertRaises(LookupError):
            g('xy-zz')

    @override_settings(WAGTAIL_CONTENT_LANGUAGES=[
        ('en', 'English'),
        ('de', 'German'),
    ])
    def test_uses_wagtail_content_languages(self):
        # be sure it's not using Django's LANGUAGES
        g = get_supported_content_language_variant
        self.assertEqual(g('en'), 'en')
        self.assertEqual(g('en-gb'), 'en')
        self.assertEqual(g('de'), 'de')
        self.assertEqual(g('de-at'), 'de')
        self.assertEqual(g('de-ch'), 'de')
        with self.assertRaises(LookupError):
            g('pt-br')
        with self.assertRaises(LookupError):
            g('pt')
        with self.assertRaises(LookupError):
            g('pt-pt')
        with self.assertRaises(LookupError):
            g('xyz')
        with self.assertRaises(LookupError):
            g('xy-zz')


@override_settings(
    USE_I18N=False,
    WAGTAIL_I18N_ENABLED=False,
    WAGTAIL_CONTENT_LANGUAGES=None,
    LANGUAGE_CODE='en-us',
)
class TestGetSupportedContentLanguageVariantWithI18nFalse(TestCase):
    def setUp(self):
        # Need to forcibly clear the django.utils.translation._trans object when overriding
        # USE_I18N:
        # https://github.com/django/django/blob/3.1/django/utils/translation/__init__.py#L46-L48
        _trans.__dict__.clear()

    def tearDown(self):
        _trans.__dict__.clear()

    def test_lookup_language_with_i18n_false(self):
        # Make sure we can handle the 'null' USE_I18N=False implementation of
        # get_supported_language_variant returning 'en-us' rather than 'en',
        # despite 'en-us' not being in LANGUAGES.
        # https://github.com/wagtail/wagtail/issues/6539

        self.assertEqual(get_supported_content_language_variant('en-us'), 'en')

    @override_settings(LANGUAGE_CODE='zz')
    def test_language_code_not_in_languages(self):
        # Ensure we can handle a LANGUAGE_CODE setting that isn't defined in LANGUAGES -
        # in this case get_content_languages has to cope with not being able to retrieve
        # a display name for the language
        self.assertEqual(get_supported_content_language_variant('zz'), 'zz')
        self.assertEqual(get_supported_content_language_variant('zz-gb'), 'zz')


class TestMultigetattr(TestCase):
    def setUp(self):

        class Thing:
            colour = 'green'
            limbs = {'arms': 2, 'legs': 3}

            def __init__(self):
                self.poke_was_called = False

            def speak(self):
                return "raaargh"

            def feed(self, food):
                return "gobble"

            def poke(self):
                self.poke_was_called = True
                raise Exception("don't do that")
            poke.alters_data = True

        self.thing = Thing()

    def test_multigetattr(self):
        self.assertEqual(multigetattr(self.thing, 'colour'), 'green')
        self.assertEqual(multigetattr(self, 'thing.colour'), 'green')
        self.assertEqual(multigetattr(self.thing, 'limbs.arms'), 2)
        self.assertEqual(multigetattr(self.thing, 'speak'), 'raaargh')
        self.assertEqual(multigetattr(self, 'thing.speak.0'), 'r')

        with self.assertRaises(AttributeError):
            multigetattr(self.thing, 'name')

        with self.assertRaises(AttributeError):
            multigetattr(self.thing, 'limbs.antennae')

        with self.assertRaises(AttributeError):
            multigetattr(self.thing, 'speak.999')

        with self.assertRaises(TypeError):
            multigetattr(self.thing, 'feed')

        with self.assertRaises(SuspiciousOperation):
            multigetattr(self.thing, 'poke')
        self.assertFalse(self.thing.poke_was_called)



from django.urls import path, re_path

from wagtail.contrib.documents.views import serve


urlpatterns = [
    re_path(r'^(\d+)/(.*)$', serve.serve, name='wagtaildocs_serve'),
    path(
        'authenticate_with_password/<int:restriction_id>/',
        serve.authenticate_with_password,
        name='wagtaildocs_authenticate_with_password'),
]



# Generated by Django 3.1.8 on 2021-06-22 15:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailsearchpromotions', '0004_copy_queries'),
    ]

    operations = [
        migrations.AlterField(
            model_name='searchpromotion',
            name='query',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='editors_picks', to='wagtailsearchpromotions.query'),
        ),
    ]



from django.contrib.admin.utils import quote, unquote
from django.core.paginator import Paginator
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.generic.base import View

from wagtail.admin.forms.search import SearchForm
from wagtail.admin.modal_workflow import render_modal_workflow
from wagtail.contrib.snippets.views.snippets import get_snippet_model_from_url_params
from wagtail.models import Locale, TranslatableMixin
from wagtail.search.backends import get_search_backend
from wagtail.search.index import class_is_indexed


class BaseChooseView(View):
    def get(self, request, app_label, model_name):
        self.model = get_snippet_model_from_url_params(app_label, model_name)

        items = self.model.objects.all()

        # Preserve the snippet's model-level ordering if specified, but fall back on PK if not
        # (to ensure pagination is consistent)
        if not items.ordered:
            items = items.order_by('pk')

        # Filter by locale
        self.locale = None
        self.locale_filter = None
        self.selected_locale = None
        if issubclass(self.model, TranslatableMixin):
            # 'locale' is the Locale of the object that this snippet is being chosen for
            if request.GET.get('locale'):
                self.locale = get_object_or_404(Locale, language_code=request.GET['locale'])

            # 'locale_filter' is the current value of the "Locale" selector in the UI
            if request.GET.get('locale_filter'):
                self.locale_filter = get_object_or_404(Locale, language_code=request.GET['locale_filter'])

            self.selected_locale = self.locale_filter or self.locale

            if self.selected_locale:
                items = items.filter(locale=self.selected_locale)

        # Search
        self.is_searchable = class_is_indexed(self.model)
        self.is_searching = False
        self.search_query = None
        if self.is_searchable and 'q' in request.GET:
            self.search_form = SearchForm(request.GET, placeholder=_("Search %(snippet_type_name)s") % {
                'snippet_type_name': self.model._meta.verbose_name
            })

            if self.search_form.is_valid():
                self.search_query = self.search_form.cleaned_data['q']

                search_backend = get_search_backend()
                items = search_backend.search(self.search_query, items)
                self.is_searching = True

        else:
            self.search_form = SearchForm(placeholder=_("Search %(snippet_type_name)s") % {
                'snippet_type_name': self.model._meta.verbose_name
            })

        # Pagination
        paginator = Paginator(items, per_page=25)
        self.paginated_items = paginator.get_page(request.GET.get('p'))

        return self.render_to_response()

    def render_to_response(self):
        raise NotImplementedError()


class ChooseView(BaseChooseView):
    # Return the choose view as a ModalWorkflow response
    def render_to_response(self):
        return render_modal_workflow(
            self.request,
            'wagtailsnippets/chooser/choose.html', None,
            {
                'model_opts': self.model._meta,
                'items': self.paginated_items,
                'is_searchable': self.is_searchable,
                'search_form': self.search_form,
                'query_string': self.search_query,
                'is_searching': self.is_searching,
                'locale': self.locale,
                'locale_filter': self.locale_filter,
                'selected_locale': self.selected_locale,
                'locale_options': Locale.objects.all() if issubclass(self.model, TranslatableMixin) else [],
            }, json_data={'step': 'choose'}
        )


class ChooseResultsView(BaseChooseView):
    # Return just the HTML fragment for the results
    def render_to_response(self):
        return TemplateResponse(self.request, "wagtailsnippets/chooser/results.html", {
            'model_opts': self.model._meta,
            'items': self.paginated_items,
            'query_string': self.search_query,
            'is_searching': self.is_searching,
        })


def chosen(request, app_label, model_name, pk):
    model = get_snippet_model_from_url_params(app_label, model_name)
    item = get_object_or_404(model, pk=unquote(pk))

    snippet_data = {
        'id': str(item.pk),
        'string': str(item),
        'edit_link': reverse('wagtailsnippets:edit', args=(
            app_label, model_name, quote(item.pk)))
    }

    return render_modal_workflow(
        request,
        None, None,
        None, json_data={'step': 'chosen', 'result': snippet_data}
    )



from django.test import TestCase

from wagtail.contrib.settings.forms import SiteSwitchForm
from wagtail.models import Page, Site
from wagtail.test.testapp.models import TestSetting


class TestSiteSwitchFromSiteOrdering(TestCase):
    def setUp(self):
        self.root_page = Page.objects.get(pk=2)
        Site.objects.all().delete()  # Drop the initial site.

    def test_site_order_by_hostname(self):
        site_1 = Site.objects.create(hostname='charly.com', root_page=self.root_page)
        site_2 = Site.objects.create(hostname='bravo.com', root_page=self.root_page, is_default_site=True)
        site_3 = Site.objects.create(hostname='alfa.com', root_page=self.root_page)
        form = SiteSwitchForm(site_1, TestSetting)
        expected_choices = [
            ('/admin/settings/tests/testsetting/{}/'.format(site_3.id), 'alfa.com'),
            ('/admin/settings/tests/testsetting/{}/'.format(site_2.id), 'bravo.com [default]'),
            ('/admin/settings/tests/testsetting/{}/'.format(site_1.id), 'charly.com')
        ]
        self.assertEqual(form.fields['site'].choices, expected_choices)



from django.apps import apps
from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.indexes import GinIndex
from django.contrib.postgres.search import SearchVectorField
from django.db import models
from django.db.models.functions import Cast
from django.utils.translation import gettext_lazy as _

from wagtail.search.index import class_is_indexed

from .utils import get_descendants_content_types_pks


class TextIDGenericRelation(GenericRelation):
    auto_created = True

    def get_content_type_lookup(self, alias, remote_alias):
        field = self.remote_field.model._meta.get_field(
            self.content_type_field_name)
        return field.get_lookup('in')(
            field.get_col(remote_alias),
            get_descendants_content_types_pks(self.model))

    def get_object_id_lookup(self, alias, remote_alias):
        from_field = self.remote_field.model._meta.get_field(
            self.object_id_field_name)
        to_field = self.model._meta.pk
        return from_field.get_lookup('exact')(
            from_field.get_col(remote_alias),
            Cast(to_field.get_col(alias), from_field))

    def get_extra_restriction(self, where_class, alias, remote_alias):
        cond = where_class()
        cond.add(self.get_content_type_lookup(alias, remote_alias), 'AND')
        cond.add(self.get_object_id_lookup(alias, remote_alias), 'AND')
        return cond

    def resolve_related_fields(self):
        return []


class IndexEntry(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    # We do not use an IntegerField since primary keys are not always integers.
    object_id = models.TextField()
    content_object = GenericForeignKey()

    # TODO: Add per-object boosting.
    autocomplete = SearchVectorField()
    title = SearchVectorField()
    # This field stores the "Title Normalisation Factor"
    # This factor is multiplied onto the the rank of the title field.
    # This allows us to apply a boost to results with shorter titles
    # elevating more specific matches to the top.
    title_norm = models.FloatField(default=1.0)
    body = SearchVectorField()

    class Meta:
        unique_together = ('content_type', 'object_id')
        verbose_name = _('index entry')
        verbose_name_plural = _('index entries')
        # An additional computed GIN index on 'title || body' is created in a SQL migration
        # covers the default case of PostgresSearchQueryCompiler.get_index_vectors.
        indexes = [GinIndex(fields=['autocomplete']),
                   GinIndex(fields=['title']),
                   GinIndex(fields=['body'])]

    def __str__(self):
        return '%s: %s' % (self.content_type.name, self.content_object)

    @property
    def model(self):
        return self.content_type.model

    @classmethod
    def add_generic_relations(cls):
        for model in apps.get_models():
            if class_is_indexed(model):
                TextIDGenericRelation(cls).contribute_to_class(model,
                                                               'postgres_index_entries')



from unittest import TestCase

from django.test import TestCase as DjangoTestCase
from django.urls import reverse

from wagtail.admin.search import SearchArea
from wagtail.admin.ui.sidebar import (
    CustomBrandingModule, LinkMenuItem, MainMenuModule, PageExplorerMenuItem, SearchModule,
    SubMenuItem, WagtailBrandingModule)
from wagtail.test.utils import WagtailTestUtils
from wagtail.utils.telepath import JSContext


class TestAdaptLinkMenuItem(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(LinkMenuItem('link', "Link", '/link/'))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.LinkMenuItem',
            '_args': [
                {
                    'classnames': '',
                    'icon_name': '',
                    'label': 'Link',
                    'name': 'link',
                    'url': '/link/'
                }
            ]
        })

    def test_adapt_with_classnames_and_icon(self):
        packed = JSContext().pack(LinkMenuItem('link', "Link", '/link/', icon_name='link-icon', classnames='some classes'))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.LinkMenuItem',
            '_args': [
                {
                    'classnames': 'some classes',
                    'icon_name': 'link-icon',
                    'label': 'Link',
                    'name': 'link',
                    'url': '/link/'
                }
            ]
        })


class TestAdaptSubMenuItem(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(
            SubMenuItem('sub-menu', "Sub menu", [
                LinkMenuItem('link', "Link", '/link/', icon_name='link-icon'),
            ], footer_text='Footer text')
        )

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.SubMenuItem',
            '_args': [
                {
                    'name': 'sub-menu',
                    'label': 'Sub menu',
                    'icon_name': '',
                    'classnames': '',
                    'footer_text': 'Footer text'
                },
                [
                    {
                        '_type': 'wagtail.sidebar.LinkMenuItem',
                        '_args': [
                            {
                                'name': 'link',
                                'label': 'Link',
                                'icon_name': 'link-icon',
                                'classnames': '',
                                'url': '/link/'
                            }
                        ]
                    }
                ]
            ]
        })

    def test_adapt_without_footer_text(self):
        packed = JSContext().pack(
            SubMenuItem('sub-menu', "Sub menu", [
                LinkMenuItem('link', "Link", '/link/', icon_name='link-icon'),
            ])
        )

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.SubMenuItem',
            '_args': [
                {
                    'name': 'sub-menu',
                    'label': 'Sub menu',
                    'icon_name': '',
                    'classnames': '',
                    'footer_text': ''
                },
                [
                    {
                        '_type': 'wagtail.sidebar.LinkMenuItem',
                        '_args': [
                            {
                                'name': 'link',
                                'label': 'Link',
                                'icon_name': 'link-icon',
                                'classnames': '',
                                'url': '/link/'
                            }
                        ]
                    }
                ]
            ]
        })


class TestAdaptPageExplorerMenuItem(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(PageExplorerMenuItem('pages', "Pages", '/pages/', 1))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.PageExplorerMenuItem',
            '_args': [
                {
                    'classnames': '',
                    'icon_name': '',
                    'label': 'Pages',
                    'name': 'pages',
                    'url': '/pages/'
                },
                1
            ]
        })


class TestAdaptWagtailBrandingModule(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(WagtailBrandingModule())

        self.assertEqual(packed['_type'], 'wagtail.sidebar.WagtailBrandingModule')
        self.assertEqual(len(packed['_args']), 2)
        self.assertEqual(packed['_args'][0], reverse('wagtailadmin_home'))
        self.assertEqual(packed['_args'][1].keys(), {
            'desktopLogoBody',
            'desktopLogoEyeClosed',
            'desktopLogoEyeOpen',
            'desktopLogoTail',
            'mobileLogo'
        })


class TestAdaptCustomBrandingModule(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(CustomBrandingModule('<h1>My custom branding</h1>'))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.CustomBrandingModule',
            '_args': [
                '<h1>My custom branding</h1>',
                False
            ]
        })

    def test_collapsible(self):
        packed = JSContext().pack(CustomBrandingModule('<h1>My custom branding</h1>', collapsible=True))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.CustomBrandingModule',
            '_args': [
                '<h1>My custom branding</h1>',
                True
            ]
        })


class TestAdaptSearchModule(TestCase):
    def test_adapt(self):
        packed = JSContext().pack(SearchModule(SearchArea("Search", '/search/')))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.SearchModule',
            '_args': [
                '/search/'
            ]
        })


class TestAdaptMainMenuModule(DjangoTestCase, WagtailTestUtils):
    def test_adapt(self):
        main_menu = [
            LinkMenuItem('pages', "Pages", '/pages/'),
        ]
        account_menu = [
            LinkMenuItem('account', "Account", reverse('wagtailadmin_account'), icon_name='user'),
            LinkMenuItem('logout', "Logout", reverse('wagtailadmin_logout'), icon_name='logout'),
        ]
        user = self.create_user(username='admin')

        packed = JSContext().pack(MainMenuModule(main_menu, account_menu, user))

        self.assertEqual(packed, {
            '_type': 'wagtail.sidebar.MainMenuModule',
            '_args': [
                [
                    {
                        '_type': 'wagtail.sidebar.LinkMenuItem',
                        '_args': [
                            {'name': 'pages', 'label': 'Pages', 'icon_name': '', 'classnames': '', 'url': '/pages/'}
                        ]
                    }
                ],
                [
                    {
                        '_type': 'wagtail.sidebar.LinkMenuItem',
                        '_args': [
                            {'name': 'account', 'label': 'Account', 'icon_name': 'user', 'classnames': '', 'url': reverse('wagtailadmin_account')}
                        ]
                    },
                    {
                        '_type': 'wagtail.sidebar.LinkMenuItem',
                        '_args': [
                            {'name': 'logout', 'label': 'Logout', 'icon_name': 'logout', 'classnames': '', 'url': reverse('wagtailadmin_logout')}
                        ]
                    }
                ],
                {
                    'name': user.first_name or user.get_username(),
                    'avatarUrl': '//www.gravatar.com/avatar/e64c7d89f26bd1972efa854d13d7dd61?s=100&d=mm'
                }
            ]
        })



from django.template.loader import render_to_string
from django.utils.functional import cached_property

from wagtail.admin.compare import BlockComparison
from wagtail.blocks import ChooserBlock

from .shortcuts import get_rendition_or_not_found


class ImageChooserBlock(ChooserBlock):
    @cached_property
    def target_model(self):
        from wagtail.contrib.images import get_image_model
        return get_image_model()

    @cached_property
    def widget(self):
        from wagtail.contrib.images.widgets import AdminImageChooser
        return AdminImageChooser()

    def get_form_state(self, value):
        value_data = self.widget.get_value_data(value)
        if value_data is None:
            return None
        else:
            return {
                'id': value_data['id'],
                'edit_link': value_data['edit_url'],
                'title': value_data['title'],
                'preview': value_data['preview'],
            }

    def render_basic(self, value, context=None):
        if value:
            return get_rendition_or_not_found(value, 'original').img_tag()
        else:
            return ''

    def get_comparison_class(self):
        return ImageChooserBlockComparison

    class Meta:
        icon = "image"


class ImageChooserBlockComparison(BlockComparison):
    def htmlvalue(self, val):
        return render_to_string("wagtailimages/widgets/compare.html", {
            'image_a': val,
            'image_b': val,
        })

    def htmldiff(self):
        return render_to_string("wagtailimages/widgets/compare.html", {
            'image_a': self.val_a,
            'image_b': self.val_b,
        })



from datetime import timedelta
from unittest import mock

from django.conf import settings
from django.template import Context, Template
from django.test import TestCase
from django.test.utils import override_settings
from django.utils import timezone
from django.utils.html import format_html
from freezegun import freeze_time

from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.ui.components import Component
from wagtail.contrib.images.tests.utils import get_test_image_file
from wagtail.models import UserProfile
from wagtail.templatetags.wagtailadmin import (
    avatar_url, notification_static, timesince_last_update, timesince_simple)
from wagtail.test.utils import WagtailTestUtils


class TestAvatarTemplateTag(TestCase, WagtailTestUtils):
    def setUp(self):
        # Create a user
        self.test_user = self.create_user(
            username='testuser',
            email='testuser@email.com',
            password='password',
        )

    def test_use_gravatar_by_default(self):
        url = avatar_url(self.test_user)
        self.assertIn('www.gravatar.com', url)

    def test_skip_gravatar_if_no_email(self):
        self.test_user.email = ''
        url = avatar_url(self.test_user)
        self.assertIn('default-user-avatar', url)

    @override_settings(WAGTAIL_GRAVATAR_PROVIDER_URL='https://robohash.org')
    def test_custom_gravatar_provider(self):
        url = avatar_url(self.test_user)
        self.assertIn('robohash.org', url)

    @override_settings(WAGTAIL_GRAVATAR_PROVIDER_URL=None)
    def test_disable_gravatar(self):
        url = avatar_url(self.test_user)
        self.assertIn('default-user-avatar', url)

    def test_uploaded_avatar(self):
        user_profile = UserProfile.get_for_user(self.test_user)
        user_profile.avatar = get_test_image_file(filename='custom-avatar.png')
        user_profile.save()

        url = avatar_url(self.test_user)
        self.assertIn('custom-avatar', url)


class TestNotificationStaticTemplateTag(TestCase):
    @override_settings(STATIC_URL='/static/')
    def test_local_notification_static(self):
        url = notification_static('wagtailadmin/images/email-header.jpg')
        self.assertEqual('{}/static/wagtailadmin/images/email-header.jpg'.format(settings.BASE_URL), url)

    @override_settings(STATIC_URL='/static/', BASE_URL='http://localhost:8000')
    def test_local_notification_static_baseurl(self):
        url = notification_static('wagtailadmin/images/email-header.jpg')
        self.assertEqual('http://localhost:8000/static/wagtailadmin/images/email-header.jpg', url)

    @override_settings(STATIC_URL='https://s3.amazonaws.com/somebucket/static/', BASE_URL='http://localhost:8000')
    def test_remote_notification_static(self):
        url = notification_static('wagtailadmin/images/email-header.jpg')
        self.assertEqual('https://s3.amazonaws.com/somebucket/static/wagtailadmin/images/email-header.jpg', url)


class TestVersionedStatic(TestCase):
    def test_versioned_static(self):
        result = versioned_static('wagtailadmin/js/core.js')
        self.assertRegex(result, r'^/static/wagtailadmin/js/core.js\?v=(\w+)$')

    @mock.patch('wagtail.admin.staticfiles.static')
    def test_versioned_static_version_string(self, mock_static):
        mock_static.return_value = '/static/wagtailadmin/js/core.js?v=123'
        result = versioned_static('wagtailadmin/js/core.js')
        self.assertEqual(result, '/static/wagtailadmin/js/core.js?v=123')
        mock_static.assert_called_once_with('wagtailadmin/js/core.js')

    def test_versioned_static_absolute_path(self):
        result = versioned_static('/static/wagtailadmin/js/core.js')
        self.assertEqual(result, '/static/wagtailadmin/js/core.js')

    def test_versioned_static_url(self):
        result = versioned_static('http://example.org/static/wagtailadmin/js/core.js')
        self.assertEqual(result, 'http://example.org/static/wagtailadmin/js/core.js')


@freeze_time("2020-07-01 12:00:00")
class TestTimesinceTags(TestCase):
    def test_timesince_simple(self):
        now = timezone.now()
        ts = timesince_simple(now)
        self.assertEqual(ts, "Just now")

        ts = timesince_simple(now - timedelta(hours=1, minutes=10))
        self.assertEqual(ts, "1\xa0hour ago")

        ts = timesince_simple(now - timedelta(weeks=2, hours=1, minutes=10))
        self.assertEqual(ts, "2\xa0weeks ago")

    def test_timesince_last_update_today_shows_time(self):
        dt = timezone.now() - timedelta(hours=1)
        formatted_time = dt.astimezone(timezone.get_current_timezone()).strftime('%H:%M')

        timesince = timesince_last_update(dt)
        self.assertEqual(timesince, formatted_time)

        # Check prefix output
        timesince = timesince_last_update(dt, time_prefix='my prefix')
        self.assertEqual(timesince, 'my prefix {}'.format(formatted_time))

    def test_timesince_last_update_before_today_shows_timeago(self):
        dt = timezone.now() - timedelta(weeks=1, days=2)

        timesince = timesince_last_update(dt, use_shorthand=False)
        self.assertEqual(timesince, '1\xa0week, 2\xa0days ago')

        timesince = timesince_last_update(dt)
        self.assertEqual(timesince, '1\xa0week ago')


class TestComponentTag(TestCase):
    def test_passing_context_to_component(self):
        class MyComponent(Component):
            def render_html(self, parent_context):
                return format_html("<h1>{} was here</h1>", parent_context.get('first_name'))

        template = Template(
            "{% load wagtailadmin_tags %}{% with first_name='Kilroy' %}{% component my_component %}{% endwith %}"
        )
        html = template.render(Context({'my_component': MyComponent()}))
        self.assertEqual(html, "<h1>Kilroy was here</h1>")

    def test_component_escapes_unsafe_strings(self):
        class MyComponent(Component):
            def render_html(self, parent_context):
                return "Look, I'm running with scissors! 8< 8< 8<"

        template = Template(
            "{% load wagtailadmin_tags %}<h1>{% component my_component %}</h1>"
        )
        html = template.render(Context({'my_component': MyComponent()}))
        self.assertEqual(html, "<h1>Look, I&#x27;m running with scissors! 8&lt; 8&lt; 8&lt;</h1>")

    def test_error_on_rendering_non_component(self):
        template = Template(
            "{% load wagtailadmin_tags %}<h1>{% component my_component %}</h1>"
        )

        with self.assertRaises(ValueError) as cm:
            template.render(Context({'my_component': "hello"}))
        self.assertEqual(str(cm.exception), "Cannot render 'hello' as a component")



from django.urls import path

from . import views


app_name = 'wagtailsettings'
urlpatterns = [
    path('<slug:app_name>/<slug:model_name>/', views.edit_current_site, name='edit'),
    path('<slug:app_name>/<slug:model_name>/<int:site_pk>/', views.edit, name='edit'),
]



from django import template

from .wagtail import (
    include_block, pageurl, richtext, slugurl, wagtail_documentation_path,
    wagtail_release_notes_path, wagtail_site, wagtail_version)


register = template.Library()

register.simple_tag(takes_context=True)(pageurl)
register.simple_tag(takes_context=True)(slugurl)
register.simple_tag(wagtail_version)
register.simple_tag(wagtail_documentation_path)
register.simple_tag(wagtail_release_notes_path)
register.filter(richtext)
register.tag(include_block)
register.simple_tag(takes_context=True)(wagtail_site)



from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator
from django.http import Http404
from django.http.request import QueryDict
from django.template.response import TemplateResponse
from django.views.decorators.vary import vary_on_headers

from wagtail.admin.auth import user_has_any_page_permission, user_passes_test
from wagtail.admin.forms.search import SearchForm
from wagtail.models import Page
from wagtail.search.query import MATCH_ALL
from wagtail.search.utils import parse_query_string


def page_filter_search(q, pages, all_pages=None, ordering=None):
    # Parse query
    filters, query = parse_query_string(q, operator='and', zero_terms=MATCH_ALL)

    # Live filter
    live_filter = filters.get('live') or filters.get('published')
    live_filter = live_filter and live_filter.lower()

    if live_filter in ['yes', 'true']:
        if all_pages is not None:
            all_pages = all_pages.filter(live=True)
        pages = pages.filter(live=True)
    elif live_filter in ['no', 'false']:
        if all_pages is not None:
            all_pages = all_pages.filter(live=False)
        pages = pages.filter(live=False)

    # Search
    if all_pages is not None:
        all_pages = all_pages.search(query, order_by_relevance=not ordering)
    pages = pages.search(query, order_by_relevance=not ordering)

    return pages, all_pages


@vary_on_headers('X-Requested-With')
@user_passes_test(user_has_any_page_permission)
def search(request):
    pages = all_pages = Page.objects.all().prefetch_related('content_type').specific()
    q = MATCH_ALL
    content_types = []
    pagination_query_params = QueryDict({}, mutable=True)
    ordering = None

    if 'ordering' in request.GET:
        if request.GET['ordering'] in ['title', '-title', 'latest_revision_created_at', '-latest_revision_created_at', 'live', '-live']:
            ordering = request.GET['ordering']

            if ordering == 'title':
                pages = pages.order_by('title')
            elif ordering == '-title':
                pages = pages.order_by('-title')

            if ordering == 'latest_revision_created_at':
                pages = pages.order_by('latest_revision_created_at')
            elif ordering == '-latest_revision_created_at':
                pages = pages.order_by('-latest_revision_created_at')

            if ordering == 'live':
                pages = pages.order_by('live')
            elif ordering == '-live':
                pages = pages.order_by('-live')

    if 'content_type' in request.GET:
        pagination_query_params['content_type'] = request.GET['content_type']

        try:
            app_label, model_name = request.GET['content_type'].split('.')
        except ValueError:
            raise Http404

        try:
            selected_content_type = ContentType.objects.get_by_natural_key(app_label, model_name)
        except ContentType.DoesNotExist:
            raise Http404

        pages = pages.filter(content_type=selected_content_type)
    else:
        selected_content_type = None

    if 'q' in request.GET:
        form = SearchForm(request.GET)
        if form.is_valid():
            q = form.cleaned_data['q']
            pagination_query_params['q'] = q

            # Parse query and filter
            pages, all_pages = page_filter_search(q, pages, all_pages, ordering)

            # Facets
            if pages.supports_facet:
                content_types = [
                    (ContentType.objects.get(id=content_type_id), count)
                    for content_type_id, count in all_pages.facet('content_type_id').items()
                ]

    else:
        form = SearchForm()

    paginator = Paginator(pages, per_page=20)
    pages = paginator.get_page(request.GET.get('p'))

    if request.headers.get('x-requested-with') == 'XMLHttpRequest':
        return TemplateResponse(request, "wagtailadmin/pages/search_results.html", {
            'pages': pages,
            'all_pages': all_pages,
            'query_string': q,
            'content_types': content_types,
            'selected_content_type': selected_content_type,
            'ordering': ordering,
            'pagination_query_params': pagination_query_params.urlencode(),
        })
    else:
        return TemplateResponse(request, "wagtailadmin/pages/search.html", {
            'search_form': form,
            'pages': pages,
            'all_pages': all_pages,
            'query_string': q,
            'content_types': content_types,
            'selected_content_type': selected_content_type,
            'ordering': ordering,
            'pagination_query_params': pagination_query_params.urlencode(),
        })



import datetime

from functools import wraps
from unittest import mock

from django.test import TestCase
from django.urls import reverse
from django.utils import timezone
from freezegun import freeze_time

from wagtail.admin.views.pages.preview import PreviewOnEdit
from wagtail.edit_handlers import FieldPanel, ObjectList, TabbedInterface
from wagtail.models import Page
from wagtail.test.testapp.models import EventCategory, EventPage, SimplePage, StreamPage
from wagtail.test.utils import WagtailTestUtils


class TestIssue2599(TestCase, WagtailTestUtils):
    """
    When previewing a page on creation, we need to assign it a path value consistent with its
    (future) position in the tree. The naive way of doing this is to give it an index number
    one more than numchild - however, index numbers are not reassigned on page deletion, so
    this can result in a path that collides with an existing page (which is invalid).
    """

    def test_issue_2599(self):
        homepage = Page.objects.get(id=2)

        child1 = Page(title='child1')
        homepage.add_child(instance=child1)
        child2 = Page(title='child2')
        homepage.add_child(instance=child2)

        child1.delete()

        self.login()
        post_data = {
            'title': "New page!",
            'content': "Some content",
            'slug': 'hello-world',
            'action-submit': "Submit",
        }
        preview_url = reverse('wagtailadmin_pages:preview_on_add',
                              args=('tests', 'simplepage', homepage.id))
        response = self.client.post(preview_url, post_data)

        # Check the JSON response
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), {'is_valid': True})

        response = self.client.get(preview_url)

        # Check the HTML response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'tests/simple_page.html')
        self.assertContains(response, "New page!")

        # Check that the treebeard attributes were set correctly on the page object
        self.assertEqual(response.context['self'].depth, homepage.depth + 1)
        self.assertTrue(response.context['self'].path.startswith(homepage.path))
        self.assertEqual(response.context['self'].get_parent(), homepage)


def clear_edit_handler(page_cls):
    def decorator(fn):
        @wraps(fn)
        def decorated(*args, **kwargs):
            # Clear any old EditHandlers generated
            page_cls.get_edit_handler.cache_clear()
            try:
                fn(*args, **kwargs)
            finally:
                # Clear the bad EditHandler generated just now
                page_cls.get_edit_handler.cache_clear()
        return decorated
    return decorator


class TestPreview(TestCase, WagtailTestUtils):
    fixtures = ['test.json']

    def setUp(self):
        self.meetings_category = EventCategory.objects.create(name='Meetings')
        self.parties_category = EventCategory.objects.create(name='Parties')
        self.holidays_category = EventCategory.objects.create(name='Holidays')

        self.home_page = Page.objects.get(url_path='/home/')
        self.event_page = Page.objects.get(url_path='/home/events/christmas/')

        self.user = self.login()

        self.post_data = {
            'title': "Beach party",
            'slug': 'beach-party',
            'body': '''{"entityMap": {},"blocks": [
                {"inlineStyleRanges": [], "text": "party on wayne", "depth": 0, "type": "unstyled", "key": "00000", "entityRanges": []}
            ]}''',
            'date_from': '2017-08-01',
            'audience': 'public',
            'location': 'the beach',
            'cost': 'six squid',
            'carousel_items-TOTAL_FORMS': 0,
            'carousel_items-INITIAL_FORMS': 0,
            'carousel_items-MIN_NUM_FORMS': 0,
            'carousel_items-MAX_NUM_FORMS': 0,
            'speakers-TOTAL_FORMS': 0,
            'speakers-INITIAL_FORMS': 0,
            'speakers-MIN_NUM_FORMS': 0,
            'speakers-MAX_NUM_FORMS': 0,
            'related_links-TOTAL_FORMS': 0,
            'related_links-INITIAL_FORMS': 0,
            'related_links-MIN_NUM_FORMS': 0,
            'related_links-MAX_NUM_FORMS': 0,
            'head_counts-TOTAL_FORMS': 0,
            'head_counts-INITIAL_FORMS': 0,
            'head_counts-MIN_NUM_FORMS': 0,
            'head_counts-MAX_NUM_FORMS': 0,
            'categories': [self.parties_category.id, self.holidays_category.id],
            'comments-TOTAL_FORMS': 0,
            'comments-INITIAL_FORMS': 0,
            'comments-MIN_NUM_FORMS': 0,
            'comments-MAX_NUM_FORMS': 1000,
        }

    def test_preview_on_create_with_m2m_field(self):
        preview_url = reverse('wagtailadmin_pages:preview_on_add',
                              args=('tests', 'eventpage', self.home_page.id))
        response = self.client.post(preview_url, self.post_data)

        # Check the JSON response
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), {'is_valid': True})

        # Check the user can refresh the preview
        preview_session_key = 'wagtail-preview-tests-eventpage-{}'.format(self.home_page.id)
        self.assertTrue(preview_session_key in self.client.session)

        response = self.client.get(preview_url)

        # Check the HTML response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'tests/event_page.html')
        self.assertContains(response, "Beach party")
        self.assertContains(response, "<li>Parties</li>")
        self.assertContains(response, "<li>Holidays</li>")

    def test_preview_on_edit_with_m2m_field(self):
        preview_url = reverse('wagtailadmin_pages:preview_on_edit',
                              args=(self.event_page.id,))
        response = self.client.post(preview_url, self.post_data)

        # Check the JSON response
        self.assertEqual(response.status_code, 200)
        self.assertJSONEqual(response.content.decode(), {'is_valid': True})

        # Check the user can refresh the preview
        preview_session_key = 'wagtail-preview-{}'.format(self.event_page.id)
        self.assertTrue(preview_session_key in self.client.session)

        response = self.client.get(preview_url)

        # Check the HTML response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'tests/event_page.html')
        self.assertContains(response, "Beach party")
        self.assertContains(response, "<li>Parties</li>")
        self.assertContains(response, "<li>Holidays</li>")

    def test_preview_on_edit_expiry(self):
        initial_datetime = timezone.now()
        expiry_datetime = initial_datetime + datetime.timedelta(
            seconds=PreviewOnEdit.preview_expiration_timeout + 1)

        with freeze_time(initial_datetime) as frozen_datetime:
            preview_url = reverse('wagtailadmin_pages:preview_on_edit',
                                  args=(self.event_page.id,))
            response = self.client.post(preview_url, self.post_data)

            # Check the JSON response
            self.assertEqual(response.status_code, 200)

            response = self.client.get(preview_url)

            # Check the HTML response
            self.assertEqual(response.status_code, 200)

            frozen_datetime.move_to(expiry_datetime)

            preview_url = reverse('wagtailadmin_pages:preview_on_edit',
                                  args=(self.home_page.id,))
            response = self.client.post(preview_url, self.post_data)
            self.assertEqual(response.status_code, 200)
            response = self.client.get(preview_url)
            self.assertEqual(response.status_code, 200)

    @clear_edit_handler(EventPage)
    def test_preview_with_custom_edit_handler(self):
        """
        The test is based on TestPreview.test_preview_on_create_with_m2m_field, except that the "categories"
        FieldPanel is only visible to superusers. Non-superusers should not be able to set "categories" for
        the preview.
        """

        class SuperuserEventCategoriesObjectList(ObjectList):
            def on_request_bound(self):
                new_children = []
                for child in self.children:
                    # skip the "categories" FieldPanel for non-superusers
                    if isinstance(child, FieldPanel) and child.field_name == "categories" and not self.request.user.is_superuser:
                        continue

                    new_child = child.bind_to(
                        model=self.model,
                        instance=self.instance,
                        request=self.request,
                        form=self.form,
                    )
                    new_children.append(new_child)
                self.children = new_children

        new_tabbed_interface = TabbedInterface([
            SuperuserEventCategoriesObjectList(EventPage.content_panels),
            ObjectList(EventPage.promote_panels),
        ])

        with mock.patch.object(EventPage, 'edit_handler', new=new_tabbed_interface, create=True):
            # Non-superusers should not see categories panel, so even though "post_data" contains "categories",
            # it should not be considered for the preview request.
            self.login(username='siteeditor', password='password')

            preview_url = reverse('wagtailadmin_pages:preview_on_add',
                                  args=('tests', 'eventpage', self.home_page.id))
            response = self.client.post(preview_url, self.post_data)

            # Check the JSON response
            self.assertEqual(response.status_code, 200)
            self.assertJSONEqual(response.content.decode(), {'is_valid': True})

            # Check the user can refresh the preview
            preview_session_key = 'wagtail-preview-tests-eventpage-{}'.format(self.home_page.id)
            self.assertTrue(preview_session_key in self.client.session)

            response = self.client.get(preview_url)

            # Check the HTML response
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'tests/event_page.html')
            self.assertContains(response, "Beach party")
            self.assertNotContains(response, "<li>Parties</li>")
            self.assertNotContains(response, "<li>Holidays</li>")

            # Since superusers see the "categories" panel, the posted data should be used for the preview.
            self.login(username='superuser', password='password')
            response = self.client.post(preview_url, self.post_data)

            # Check the JSON response
            self.assertEqual(response.status_code, 200)
            self.assertJSONEqual(response.content.decode(), {'is_valid': True})

            # Check the user can refresh the preview
            preview_session_key = 'wagtail-preview-tests-eventpage-{}'.format(self.home_page.id)
            self.assertTrue(preview_session_key in self.client.session)

            response = self.client.get(preview_url)

            # Check the HTML response
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'tests/event_page.html')
            self.assertContains(response, "Beach party")
            self.assertContains(response, "<li>Parties</li>")
            self.assertContains(response, "<li>Holidays</li>")


class TestDisablePreviewButton(TestCase, WagtailTestUtils):
    """
    Test that preview button can be disabled by setting preview_modes to an empty list
    """
    def setUp(self):
        # Find root page
        self.root_page = Page.objects.get(id=2)

        # Login
        self.user = self.login()

    def test_disable_preview_on_create(self):
        # preview button is available by default
        response = self.client.get(reverse('wagtailadmin_pages:add', args=('tests', 'simplepage', self.root_page.id)))
        self.assertEqual(response.status_code, 200)

        preview_url = reverse('wagtailadmin_pages:preview_on_add', args=('tests', 'simplepage', self.root_page.id))
        self.assertContains(response, '<li class="preview">')
        self.assertContains(response, 'data-action="%s"' % preview_url)

        # StreamPage has preview_modes = []
        response = self.client.get(reverse('wagtailadmin_pages:add', args=('tests', 'streampage', self.root_page.id)))
        self.assertEqual(response.status_code, 200)

        preview_url = reverse('wagtailadmin_pages:preview_on_add', args=('tests', 'streampage', self.root_page.id))
        self.assertNotContains(response, '<li class="preview">')
        self.assertNotContains(response, 'data-action="%s"' % preview_url)

    def test_disable_preview_on_edit(self):
        simple_page = SimplePage(title='simple page', content="hello")
        self.root_page.add_child(instance=simple_page)

        # preview button is available by default
        response = self.client.get(reverse('wagtailadmin_pages:edit', args=(simple_page.id, )))
        self.assertEqual(response.status_code, 200)

        preview_url = reverse('wagtailadmin_pages:preview_on_edit', args=(simple_page.id, ))
        self.assertContains(response, '<li class="preview">')
        self.assertContains(response, 'data-action="%s"' % preview_url)

        stream_page = StreamPage(title='stream page', body=[('text', 'hello')])
        self.root_page.add_child(instance=stream_page)

        # StreamPage has preview_modes = []
        response = self.client.get(reverse('wagtailadmin_pages:edit', args=(stream_page.id, )))
        self.assertEqual(response.status_code, 200)

        preview_url = reverse('wagtailadmin_pages:preview_on_edit', args=(stream_page.id, ))
        self.assertNotContains(response, '<li class="preview">')
        self.assertNotContains(response, 'data-action="%s"' % preview_url)

    def test_disable_preview_on_revisions_list(self):
        simple_page = SimplePage(title='simple page', content="hello")
        self.root_page.add_child(instance=simple_page)
        simple_page.save_revision()

        # check preview shows up by default
        response = self.client.get(reverse('wagtailadmin_pages:revisions_index', args=(simple_page.id,)))
        preview_url = reverse('wagtailadmin_pages:revisions_view', args=(simple_page.id, simple_page.get_latest_revision().id))
        self.assertContains(response, 'Preview')
        self.assertContains(response, preview_url)

        stream_page = StreamPage(title='stream page', body=[('text', 'hello')])
        self.root_page.add_child(instance=stream_page)
        latest_revision = stream_page.save_revision()

        # StreamPage has preview_modes = []
        response = self.client.get(reverse('wagtailadmin_pages:revisions_index', args=(stream_page.id,)))
        preview_url = reverse('wagtailadmin_pages:revisions_view', args=(stream_page.id, latest_revision.id))
        self.assertNotContains(response, 'Preview')
        self.assertNotContains(response, preview_url)

    def disable_preview_in_moderation_list(self):
        stream_page = StreamPage(title='stream page', body=[('text', 'hello')])
        self.root_page.add_child(instance=stream_page)
        latest_revision = stream_page.save_revision(user=self.user, submitted_for_moderation=True)

        response = self.client.get(reverse('wagtailadmin_home'))
        preview_url = reverse('wagtailadmin_pages:preview_for_moderation', args=(latest_revision.id,))
        self.assertNotContains(response, '<li class="preview">')
        self.assertNotContains(response, 'data-action="%s"' % preview_url)
